// Bibliothèques appelées
  #include <WiFiClientSecure.h>
  #include <NTPClient.h>
  #include <WiFiUdp.h>
  #include <TM1637Display.h>
  #include <SPI.h>
  #include <SD.h>
  #include "identifiants.h"                                     //définition des paramètres réseau à compléter dans le fichier

//définition des pins utilisées
/*******************************************************************************************
 * Pour le WEMOS LOLIN32, on peut utiliser le port série qui est sur les broches 16 et 17  *
 * Connexter RX de ECH sur TX de arduino et TX de ECH sur RX de ARDUINO                    *
 * et bien sur les 0v ensembles pour qu'ils aient la même référence                        *
 ******************************************************************************************/
  #define arduino_TX 16                                         //Pin déclarée pour savoir son utilité
  #define arduino_RX 17                                         //Pin déclarée pour savoir son utilité
  
// Afficheur 7 digits
  #define CLK 14                                                //Pins for TM1637       
  #define DIO 27                                                //Pins for TM1637

// CARTE sd
  #define sck_sd 18                                             // carte SD sck
  #define miso_sd 19                                            // carte SD MISO
  #define mosi_sd 23                                            // carte SD MOSI
  #define cs_sd 5                                               // carte SD CS
   
  int analogPin1 = 32;                                          // Sonde de température 1 sur borne 32 sonde ext souflage ventil.
  int analogPin2 = 33;                                          // Sonde de température 2 sur borne 33 sonde condens
  int analogPin3 = 25;                                          // Sonde de température 3 sur borne 25
  int analogPin4 = 35;                                          // Sonde de température 4 sur borne 35


//définition des adresses à lire ou écrire dans l'ECH
/************************************************************************************
 *  indiquer les identifiants et mot de passe du wifi et du serveur FTP             *
 ************************************************************************************/
  #define temp_SD1 1135         // adresse de lecture de la température de l'eau entrant
  #define temp_SD2 1137         // adresse de lecture de la température de l'eau sortant
  #define temp_SD3 1139         // adresse de lecture de la température du condenseur
  #define temp_SD4 1141         // adresse de lecture de la température de air extérieur
  #define ad_inputs 1124        // adresse de lecture des entrées digitales
  #define ad_output 1189        // adresse de lecture des sorties digitales
  #define ad_vit_vent 1205      // adresse de lecture de la vitesse ventillateur
  #define ad_alarmes 1214       // adresse de lecture des alarmes automatiques

/*    Poids fort
 *    n_bit Label       Meaning                                     active at
      Bit_0 M_AONOFF    ON-OFF remote                                   1
      Bit_1 M_ADALTA    high pressure digital alarm                     1
      Bit_2 M_ADBASSA   low pressure digital alarm                      1
      Bit_3 M_AFLUSS    flow meter digital alarm                        1
      Bit_4 M_ATFAN     fan thermal digital alarm                       1
      Bit_5 M_AGELIN    internal circuit antifreeze analogue alarm      1
      Bit_6 M_AGELOT    external circuit antifreeze analogue alarm      1
      Bit_7 M_ANALTA    high pressure analogue alarm                    1

      Poids faible
      n_bit Label       Meaning                                      active at
      Bit_0 M_ANBASSA   low pressure analogue alarm                     1
      Bit_1 M_ASCAR     machine discharged alarm                        1
      Bit_2 M_ACONF     configuration alarm                             1
      Bit_3 M_AST1      probe 1 alarm                                   1
      Bit_4 M_AST2      probe 2 alarm                                   1
      Bit_5 M_AST3      probe 3 alarm                                   1
      Bit_6 M_AST4      probe 4 alarm                                   1
      Bit_7 M_OVERH     control temperature high analogue alarm         1  
*/
  #define ad_alarmesa 1213       // adresse de lecture des alarmes manuelles

  
  #define MASK_DO_COMPRESSOR   0x01
  #define MASK_DO_PUMP         0x02
  #define MASK_DO_REVERSAL     0x04
  #define MASK_DO_BOILER       0x08
  #define MASK_DO_ALARM        0x10



 // définition des variables globales
  #define TIMEOUT 1000
  int Heure1;
  int Heure2;
  int temp1 = 0;           
  int temp2 = 0;           
  int temp3 = 0;           
  int temp4 = 0;           
  int touslescinq;
  float Temp_SA1;               // Sonde analogique température extérieure derrière la PAC
  float Temp_SA2;               // Sonde analogique température évaporateur aval
  float Temp_SA3;               // Sonde analogique température compresseur
  float Temp_SA4;               // Sonde analogique prévue pour capteur de pression futur
  float delta;
  int unefois = 0;  
  int nbenreg =0;

  long previousMillis = 0;
  long interval = 30000;      // nombre de millisecondes entre 2 envoi FTP
  long interval_tps = 0; 
  String oldtiti="";
  String ordre_ftp="";
  float SD1;                  // température de l'eau entrant
  float SD2;                  // température de l'eau sortant
  float SD3;                  // température du condenseur
  float SD4;                  // température de air extérieur
  int dig_input;              // lecture des entrées
  int dig_output;             // lecture des sorties
  int vit_ventil;             // lecture de la vitesse du ventilateur
  int alarmes;                 // lecture des alarmes
  
  int valeur_initiale = 0;
  
  File myFile;

// démarrage serveur temps sur internet
/************************************************************************************
 *  cette fonction permet de récupérer l'heure d'internet                           *
 ************************************************************************************/
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "europe.pool.ntp.org", 3600, 60000);

// démarrage afficheur 7 digits
TM1637Display display(CLK, DIO);

//______________________  Fin des déclarations _______________________________________________________________


void setup() {
  // initialise les ports série:
  Serial.begin(115200);             // pour monitoring des échanges avec l'ordinateur
  Serial2.begin(9600,SERIAL_8E1);   // paramètrage du port série connecté à l'ECH en 9600 bauds, 8 bits, 1 stop, parité paire
  delay(200);
  while (!Serial2) {                // attente port série prêt
  }
  Serial.println("Arduino Ready");
  
    Serial.print("Connecting to ");
    Serial.println(SSID);
    WiFi.begin(SSID, PASSWORD);     //connexion au WIFI
    while (WiFi.status() != WL_CONNECTED) 
    {
        delay(500);
        Serial.print(".");
    }
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
    timeClient.begin();             // démarrage serveur temps internet
    SPI.begin(sck_sd,miso_sd,mosi_sd,cs_sd); //démarrage carte SD
    SD.begin(cs_sd);
    if (!SD.begin(cs_sd)) {
      Serial.println("Echec initialisation SD!");
     }
    
    touslescinq=0;
    display.setBrightness(0x0f);    // Contraste afficheur 
}
//_________________________  Fin du setup  ____________________________________________________________


void loop() {
  int valeur, valeur_lue,hh;
  char heure[6];
  String s_heure;
    // on lit les valeurs des capteurs analogiques
     temp1 = analogRead(analogPin1);    // read the input pin température extérieure derrière la APC
     temp2 = analogRead(analogPin2);    // read the input pin Température Evaporateur
     temp3 = analogRead(analogPin3);    // read the input pin  Température soufflage
     temp4 = analogRead(analogPin4);    // read the input pin
     Temp_SA1=(float)(2550-temp1)/43;   // converti les valeurs luesen degrés 
     Temp_SA2=(float)(2550-temp2)/43;   // converti les valeurs luesen degrés
     Temp_SA3=(float)(2000-temp3)/35;   // converti les valeurs luesen degrés
     Temp_SA4=(float)(2180-temp4)/35;   // converti les valeurs luesen degrés
  
    // on interroge l'ECH pour les températures et les entrées et sorties numériques
    valeur_lue=lit_valeur(temp_SD1);                // Interrogation de la sonde SD1
    if (valeur_lue!=9999) {SD1=(float)valeur_lue/10;}      // vérification que la valeur reçue n'est pas une erreur
    valeur_lue=lit_valeur(temp_SD2);                // Interrogation de la sonde SD2
    if (valeur_lue!=9999) {SD2=(float)valeur_lue/10;}      // vérification que la valeur reçue n'est pas une erreur
    valeur_lue=lit_valeur(temp_SD3);                // Interrogation de la sonde SD3
    if (valeur_lue!=9999) {SD3=(float)valeur_lue/10;}      // vérification que la valeur reçue n'est pas une erreur
    valeur_lue=lit_valeur(temp_SD4);                // Interrogation de la sonde SD4
    if (valeur_lue!=9999) {SD4=(float)valeur_lue/10;}      // vérification que la valeur reçue n'est pas une erreur
    valeur_lue=lit_valeur(ad_inputs);               // Interrogation de l'état des entrées
    if (valeur_lue!=9999) {dig_input=highByte(valeur_lue);}   // vérification que la valeur reçue n'est pas une erreur
//    delay (100);
//    Serial2.flush();
    valeur_lue=lit_valeur(ad_output);                // Interrogation de l'état des sorties
    if (valeur_lue!=9999) {dig_output=highByte(valeur_lue);}  // vérification que la valeur reçue n'est pas une erreur
    valeur_lue=lit_valeur(ad_vit_vent);                // Interrogation de la sonde SD4
    if (valeur_lue!=9999) {vit_ventil=(float)valeur_lue/256;}      // vérification que la valeur reçue n'est pas une erreur
    valeur_lue=lit_valeur(ad_alarmes);                // Interrogation de la sonde SD4
    if (valeur_lue!=9999) {alarmes=valeur_lue;}      // vérification que la valeur reçue n'est pas une erreur
    Serial.println(SD1);  
    Serial.println(SD2);
    Serial.println(SD3);
    Serial.println(SD4);
    Serial.println(dig_input);
    Serial.println(dig_output);
     delta=SD3-Temp_SA2;
     
    //Rafraichi l'heure depuis le serveur temps
     timeClient.update();               
     s_heure = timeClient.getFormattedTime();
     s_heure.toCharArray(heure,6);
     Heure1=((heure[0]-48)*10+heure[1]-48);
   // Mise en forme du message pour compatibilité EXCEL
     String message = s_heure+";" + String(SD1,1) + ";" + String(SD2,1) + ";"+ String(SD3,1) + ";"+ String(SD4,1)+ ";"+ String(Temp_SA1,1)+ ";"+ String(Temp_SA2,1)+ ";"+ String(Temp_SA4,1)+ ";"+ String(vit_ventil)+ ";"+ String(dig_input,BIN)+ ";"+ String(dig_output,BIN)+ ";"+ String(alarmes,BIN);
   // préparation pour affichage sur afficuer 7 digits de 5 messages différents
     touslescinq++;        
     Serial.println(touslescinq);

   // Comparaison du delta température pour envoi d'un SMS le cas échéan pour avertir d'un écart trop faible si le compresseur est en marche
      
  if ((0 != (dig_output&MASK_DO_COMPRESSOR)) & (delta<3) & (SD4<4))
  {
          unefois++;
          if (unefois == 3 )
            {
            envoisms(message);
            }
  }
  else unefois = 0;
          
  if (touslescinq == 1)
    {// affichage sur 7 segments de la température SD1 tempéarture d'arrivée d'eau de chauffage
    display.showNumberDec(SD1); 
    }
  if (touslescinq == 2)
    {// affichage sur 7 segments de la température SD2 tempéarture de retour d'eau de chauffage
    display.showNumberDec(SD2); 
    }
  if (touslescinq == 3)
    {// affichage sur 7 segments de la température SD3 tempéarture de l'évaporateur
    display.showNumberDec(SD3); 
    }
  if (touslescinq == 4)
    {// affichage sur 7 segments de la température SD4 tempéarture extérieure
    display.showNumberDec(SD4); 
    }
  if (touslescinq == 5)
    {// affichage sur 7 segments de l'heure
        hh=((heure[0]-48)*10+heure[1]-48)*100+(heure[3]-48)*10+heure[4]-48;     // conversion de l'heure en nombre
    display.showNumberDecEx(hh,(0x80 >> 1), true);                              // affichage de l'heure avec les double point
    touslescinq=0;
    ecrit_sd(message);          // appel de la fonction écrire sur SD
    }
  delay(1000);
  }
